package adminkindboxreqhandler

import (
	"net/http"
	"strconv"

	params "git.gocasts.ir/ebhomengo/niki/param"
	param "git.gocasts.ir/ebhomengo/niki/param/admin/kind_box_req"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	echo "github.com/labstack/echo/v4"
)

// AssignSenderAgent godoc
// @Summary      Admin Assign Sender Agent to kindboxreq
// @Tags         Admins KindBoxReqs
// @Accept       json
// @Produce      json
// @Param 		 id path int true "KindBoxReq ID"
// @Param 		 Request body param.AssignSenderRequest true "Assign Sender Agent Request Body"
// @Success      200  {object}   param.AssignSenderResponse
// @Failure      400  {string}  "Bad request"
// @Security 	 AuthBearerAdmin
// @Router       /admins/kindboxreqs/{id}/assign-sender-agent [patch].
func (h Handler) AssignSenderAgent(c echo.Context) error {
	var req param.AssignSenderRequest

	if bErr := c.Bind(&req); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}

	id, cErr := strconv.ParseUint(c.Param("id"), 10, 64)

	if cErr != nil {
		return c.JSON(http.StatusBadRequest, errmsg.ErrorMsgInvalidInput)
	}

	req.KindBoxReqID = uint(id)

	resp, sErr := h.adminKindBoxReqSvc.AssignSenderAgent(c.Request().Context(), req)

	if sErr != nil {
		msg, code := httpmsg.Error(sErr)
		if resp.FieldErrors != nil {
			return c.JSON(code, echo.Map{
				"message": msg,
				"errors":  resp.FieldErrors,
			})
		}

		return echo.NewHTTPError(code, msg)
	}

	go h.notificationSvc.KindBoxReqAssigned(params.NotificationKindBoxReqAssigned{
		KindBoxReqID: req.KindBoxReqID,
	})

	return c.JSON(http.StatusOK, resp)
}
