package adminkindboxreqhandler

import (
	"net/http"
	"strconv"

	params "git.gocasts.ir/ebhomengo/niki/param"
	param "git.gocasts.ir/ebhomengo/niki/param/admin/kind_box_req"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	"github.com/labstack/echo/v4"
)

// Reject godoc
// @Summary      Reject a kindboxreq by admin
// @Tags         Admins KindBoxReqs
// @Accept       json
// @Produce      json
// @Param 		 id path int true "KindBoxReq id"
// @Param 		 Request body param.KindBoxReqRejectRequest true "KindBoxReq Reject Request Body"
// @Success      200  {object}  param.KindBoxReqRejectResponse
// @Failure      400  {string}  "Bad request"
// @Security 	 AuthBearerAdmin
// @Router       /admins/kindboxreqs/{id}/reject-kind-box-req [patch].
func (h Handler) Reject(c echo.Context) error {
	var req param.KindBoxReqRejectRequest
	if bErr := c.Bind(&req); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}
	num, cErr := strconv.ParseUint(c.Param("id"), 10, 64)
	if cErr != nil {
		return c.JSON(http.StatusBadRequest, errmsg.ErrorMsgInvalidInput)
	}
	req.ID = uint(num)

	resp, sErr := h.adminKindBoxReqSvc.Reject(c.Request().Context(), req)
	if sErr != nil {
		msg, code := httpmsg.Error(sErr)
		if resp.FieldErrors != nil {
			return c.JSON(code, echo.Map{
				"message": msg,
				"errors":  resp.FieldErrors,
			})
		}

		return echo.NewHTTPError(code, msg)
	}

	go h.notificationSvc.KindBoxReqRejected(params.NotificationKindBoxReqRejected{
		KindBoxReqID: req.ID,
		Description:  req.Description,
	})

	return c.JSON(http.StatusOK, resp)
}
