package benefactorkindboxreqvalidator

import (
	"context"
	"errors"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/kind_box_req"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateUpdateRequest(ctx context.Context, req param.KindBoxReqUpdateRequest) (map[string]string, error) {
	const op = "benefactorkindboxreqvalidator.ValidateUpdateRequest"

	if err := validation.ValidateStruct(&req,
		validation.Field(&req.KindBoxReqID,
			validation.Required,
			validation.By(v.doesKindBoxRequestExist(ctx)),
			validation.By(v.doesKindBoxRequestHavePendingStatus(ctx)),
			validation.By(v.doesKindBoxBelongToBenefactor(ctx, req.BenefactorID)),
		),
		validation.Field(&req.CountRequested,
			validation.Required,
			validation.Min(MinKindBoxReq),
			validation.Max(MaxKindBoxReq),
		),
		validation.Field(&req.KindBoxType,
			validation.Required,
			validation.By(v.doesTypeExist),
		),
		validation.Field(&req.DeliverAddressID,
			validation.Required,
			validation.By(v.doesBenefactorAddressExist(ctx, req.BenefactorID)),
		),
		validation.Field(&req.DeliverReferDate,
			validation.Required,
			validation.By(v.isDateValid),
		),
		validation.Field(&req.DeliverReferTimeID,
			validation.Required,
			validation.By(v.isReferTimeIDValid(ctx)),
		),
	); err != nil {
		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return fieldErrors, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(err)
	}

	// nolint
	return nil, nil
}
